/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\NVEffectsBrowser\Effects\PF_DiffuseAndSpecular
File:  Effect_Dot3DiffSpec.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/



#ifndef __SHADER_DOT3_DIRECTIONAL_H
#define __SHADER_DOT3_DIRECTIONAL_H

#include "eb_effect.h"
#include "nvdevice.h"
#include "Dot3_Util.h"
#include "MouseUI.h"
#include <string>

class Effect_PF;

/////////////////////////////////////////////////////////////////

class NVDot3DirectionalDevice : public NVDevice
{
public:
	NVDot3DirectionalDevice(LPDIRECT3DDEVICE8 pDev = NULL, Effect_PF* pDot3 = NULL)
		: NVDevice(pDev),
			m_pDot3Shader(pDot3)
	{ };

	Effect_PF* m_pDot3Shader;
};


typedef enum tageDisplayOptions
{
	DISPLAY_BUMPMAP = 0,
	DISPLAY_LIGHTVECTOR = 1,
	DISPLAY_BASETEXTURE = 2,
	DISPLAY_RESULT =3,
	DISPLAY_SPECULARMAP =4,
	DISPLAYOPTIONS_FORCEDWORD = 0xFFFFFFFF
} eDisplayOptions;


/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////


class Effect_PF : public EBEffect
{
public:
	Effect_PF();
	~Effect_PF();
	virtual void UpdateProperties();
	virtual void PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten);
	virtual HRESULT Initialize(IDirect3DDevice8* pDev);
	virtual HRESULT Free();
	virtual HRESULT Start();
	virtual HRESULT Tick(EBTimer* pTimer);
	virtual HRESULT ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehaviour, D3DFORMAT Format);
	virtual void MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y);
	virtual void MouseMove(HWND hWnd, int x, int y);
	virtual void Keyboard(DWORD dwKey, UINT nFlags, bool bDown);

	DWORD	m_dwPSH_BumpDiffSpec;
	DWORD	m_dwPSH_ShowLightVector;
	DWORD	m_dwPSH_Textured;
	DWORD	m_dwPSH_Specular;
	DWORD	m_dwPSH_Glossy;
	DWORD	m_dwPSH_TexUnlit;
	DWORD	m_dwPSH_EnvLight;


	void	ReloadTextures();
	void	ReleaseTextures();

	void	LoadTexture( LPDIRECT3DTEXTURE8 * ppTex, const EBString & filename );

	D3DXVECTOR3	m_InitialEyePos;
	D3DXVECTOR3	m_EyePos;

protected:
	friend NVDot3DirectionalDevice;
	NVDot3DirectionalDevice* m_pNVDevice;

	HRESULT SetTransform(D3DXVECTOR3 vecLightDirection);
	HRESULT DrawDirectionalLightObject(D3DXVECTOR3 vecLightDirection);
	HRESULT DrawObjects(D3DXVECTOR3 vecLightDirection);
	HRESULT CreateCube(float width, float height, float length, IDirect3DVertexBuffer8** ppVB,
							  IDirect3DIndexBuffer8 **ppIB);
	HRESULT DrawCubeEnvironment();


	NVMesh*	m_pLightMesh;
	NVMesh*	m_pKenheadMesh;
	D3DXMATRIX m_ObjectMat;
	MouseUI* m_pUI;
	bool m_bWireframe;
	bool m_bPause;
	bool m_bViewCubeMap;
	bool m_bEnvLightMap;
	bool m_bViewHair;
	

	LPDIRECT3DTEXTURE8 m_pBumpMap;
	std::string m_strBumpMapFileName;

	LPDIRECT3DTEXTURE8 m_pNormalMap;
	std::string m_strNormalMapFileName;

	LPDIRECT3DTEXTURE8 m_pBaseTexture;
	std::string m_strBaseTexFileName;

	LPDIRECT3DTEXTURE8	m_pTex_DiffSpec_IrradianceMap;
	std::string m_strIrradianceMapFileName;

	LPDIRECT3DTEXTURE8	m_pTex_White;
	LPDIRECT3DTEXTURE8	m_pTex_FlatNM;

	LPDIRECT3DCUBETEXTURE8	m_pTex_OfficeCube;
	LPDIRECT3DCUBETEXTURE8	m_pTex_OfficeCubeDiff;
	LPDIRECT3DCUBETEXTURE8	m_pTex_OfficeCubeSpec;

	IDirect3DVertexBuffer8 *m_CubeVerts;
	IDirect3DIndexBuffer8 *m_CubeIndices;

	float m_fBumpScale;
	
	DWORD m_dwNumFaces;
	DWORD m_dwNumVertices;
	DWORD m_dwNumIndices;
	
	DWORD m_dwDot3Shader;
	DWORD m_dwVSH_EnvLight;
	DWORD m_dwVSH_Glossy;
	DWORD m_dwLightShader;
	DWORD m_dwTextureShader;

	float m_CubeSize;

	eDisplayOptions m_eDisplayOption;
	float m_fAngle;
};


#endif // __SHADER_DOT3_DIRECTIONAL_H